@@ -3,6 +3,7 @@ Manipulation of textual data.
 """
 
 import collections.abc
+import copy
 import difflib
 import os
 import pathlib
@@ -10,7 +11,6 @@ import re
 import string
 import subprocess
 
-import fuzzywuzzy.fuzz
 import numpy as np
 import pandas as pd
 
@@ -19,45 +19,6 @@ from .ops import dict_to_dataframe
 """ == Basic processing of textual data ====================================================== """
 
 
-def remove_punctuation(raw_txt, rm_whitespace=False):
-    """
-    Remove punctuation from string-type data.
-
-    :param raw_txt: string-type data
-    :type raw_txt: str
-    :param rm_whitespace: whether to remove whitespace, defaults to ``False``
-    :type rm_whitespace: bool
-    :return: text with punctuation removed
-    :rtype: str
-
-    **Examples**::
-
-        >>> from pyhelpers.text import remove_punctuation
-
-        >>> raw_text = 'Hello\tworld! :-)'
-
-        >>> text = remove_punctuation(raw_text)
-        >>> print(text)
-        Hello	world
-
-        >>> text = remove_punctuation(raw_text, rm_whitespace=True)
-        >>> print(text)
-        Hello world
-    """
-
-    try:
-        txt = raw_txt.translate(str.maketrans('', '', string.punctuation))
-
-    except Exception as e:
-        print(e)
-        txt = ''.join(x for x in raw_txt if x not in string.punctuation)
-
-    if rm_whitespace:
-        txt = ' '.join(txt.split())
-
-    return txt
-
-
 def get_acronym(text, only_capitals=False, capitals_in_words=False):
     """
     Get an acronym (in capital letters) of an input text.
@@ -78,18 +39,18 @@ def get_acronym(text, only_capitals=False, capitals_in_words=False):
 
         >>> text_a = 'This is an apple.'
         >>> acron = get_acronym(text_a)
-        >>> print(acron)
-        TIAA
+        >>> acron
+        'TIAA'
 
         >>> text_b = "I'm at the University of Birmingham."
-        >>> acron =get_acronym(text_b, only_capitals=True)
-        >>> print(acron)
-        IUB
+        >>> acron = get_acronym(text_b, only_capitals=True)
+        >>> acron
+        'IUB'
 
         >>> text_c = 'There is a "ConnectionError"!'
         >>> acron = get_acronym(text_c, capitals_in_words=True)
-        >>> print(acron)
-        TCE
+        >>> acron
+        'TCE'
     """
 
     txt = remove_punctuation(text)
@@ -104,6 +65,44 @@ def get_acronym(text, only_capitals=False, capitals_in_words=False):
     return acronym
 
 
+def remove_punctuation(x, rm_whitespace=False):
+    """
+    Remove punctuation from string-type data.
+
+    :param x: raw string-type data
+    :type x: str
+    :param rm_whitespace: whether to remove whitespace, defaults to ``False``
+    :type rm_whitespace: bool
+    :return: text with punctuation removed
+    :rtype: str
+
+    **Examples**::
+
+        >>> from pyhelpers.text import remove_punctuation
+
+        >>> raw_text = 'Hello\tworld! :-)'
+
+        >>> text = remove_punctuation(raw_text)
+        >>> text
+        'Hello\tworld '
+
+        >>> text = remove_punctuation(raw_text, rm_whitespace=True)
+        >>> text
+        'Hello world'
+    """
+
+    # noinspection PyBroadException
+    try:
+        y = x.translate(str.maketrans('', '', string.punctuation))
+    except Exception:
+        y = ''.join(p for p in x if p not in string.punctuation)
+
+    if rm_whitespace:
+        y = ' '.join(y.split())
+
+    return y
+
+
 def extract_words1upper(x, join_with=None):
     """
     Extract words from a string by spliting it at occurrence of an uppercase letter.
@@ -122,19 +121,19 @@ def extract_words1upper(x, join_with=None):
 
         >>> x1 = 'NetworkWaymarks'
         >>> res = extract_words1upper(x1, join_with=' ')
-        >>> print(res)
-        Network Waymarks
+        >>> res
+        'Network Waymarks'
 
         >>> x2 = 'Retaining_Wall'
         >>> res = extract_words1upper(x2, join_with=' ')
-        >>> print(res)
-        Retaining Wall
+        >>> res
+        'Retaining Wall'
     """
 
-    x_ = remove_punctuation(x)
+    y = remove_punctuation(x)
 
     # re.sub(r"([A-Z])", r" \1", x).split()
-    extracted_words = re.findall(r'[a-zA-Z][^A-Z]*', x_)
+    extracted_words = re.findall(r'[a-zA-Z][^A-Z]*', y)
 
     if join_with:
         extracted_words = join_with.join(extracted_words)
@@ -145,131 +144,142 @@ def extract_words1upper(x, join_with=None):
 """ == Comparison of textual data ============================================================ """
 
 
-def find_similar_str(str_x, lookup_list, processor='fuzzywuzzy', ignore_punctuation=True, **kwargs):
+def find_matched_str(x, lookup_list):
     """
-    Find similar string from a list of strings.
+    Find all that are matched with a string from among a sequence of strings.
 
-    :param str_x: a string-type variable
-    :type str_x: str
+    :param x: a string-type variable
+    :type x: str
     :param lookup_list: a sequence of strings for lookup
-    :type lookup_list: list or tuple
-
-    :param processor: options include ``'fuzzywuzzy'`` (default) and ``'difflib'``
-
-        - if ``processor='fuzzywuzzy'``, the function relies on `fuzzywuzzy.fuzz.token_set_ratio`_
-        - if ``processor='difflib'``, the function relies on `difflib.get_close_matches`_
-
-    :type processor: str
-
-    :param ignore_punctuation: whether to ignore puctuations in the search for similar texts
-    :type ignore_punctuation: bool
-    :param kwargs: optional parameters of
-        `difflib.get_close_matches`_ or `fuzzywuzzy.fuzz.token_set_ratio`_
-    :return: a string-type variable that should be similar to (or the same as) ``str_x``
-    :rtype: str or None
-
-    .. _`fuzzywuzzy.fuzz.token_set_ratio`:
-        https://github.com/seatgeek/fuzzywuzzy
-    .. _`difflib.get_close_matches`:
-        https://docs.python.org/3/library/difflib.html#difflib.get_close_matches
-
-    **Examples**::
-
-        >>> from pyhelpers.text import find_similar_str
-
-        >>> x = 'Apple'
-        >>> lookup_lst = ['abc', 'aapl', 'app', 'ap', 'ape', 'apex', 'apel']
-
-        >>> str_similar = find_similar_str(x, lookup_lst)
-        >>> print(str_similar)
-        app
-
-        >>> str_similar = find_similar_str('x', lookup_lst)
-        >>> print(str_similar is None)
-        False
-
-        >>> print(str_similar)
-        apex
-
-        >>> str_similar = find_similar_str(x, lookup_lst, processor='difflib')
-        >>> print(str_similar)
-        app
-
-        >>> str_similar = find_similar_str('x', lookup_lst, processor='difflib')
-        >>> print(str_similar is None)
-        True
-    """
-
-    assert processor in ('difflib', 'fuzzywuzzy'), \
-        "Options for `processor` include \"difflib\" and \"fuzzywuzzy\"."
-
-    if processor == 'fuzzywuzzy':
-
-        l_distances = [fuzzywuzzy.fuzz.token_set_ratio(str_x, a, **kwargs) for a in lookup_list]
-
-        if sum(l_distances) == 0:
-            sim_str = None
-        else:
-            sim_str = lookup_list[l_distances.index(max(l_distances))]
-
-    elif processor == 'difflib':
-        if not ignore_punctuation:
-            str_x_ = str_x.lower()
-            lookup_list_ = {x_.lower(): x_ for x_ in lookup_list}
-        else:
-            str_x_ = remove_punctuation(str_x.lower())
-            lookup_list_ = {remove_punctuation(x_.lower()): x_ for x_ in lookup_list}
-        sim_str_ = difflib.get_close_matches(str_x_, lookup_list_.keys(), n=1, **kwargs)
-
-        if not sim_str_:
-            sim_str = None
-        else:
-            sim_str = lookup_list_[sim_str_[0]]
-
-    else:
-        sim_str = None
-
-    return sim_str
-
-
-def find_matched_str(str_x, lookup_list):
-    """
-    Find from a list the closest, case-insensitive, str to the given one.
-
-    :param str_x: a string-type variable; if ``None``, the function will return ``None``
-    :type str_x: str or None
-    :param lookup_list: a sequence of strings for lookup
-    :type lookup_list: typing.Iterable
-    :return: a string-type variable that is case-insensitively the same as ``str_x``
-    :rtype: typing.Generator[typing.Iterable] or None
+    :type lookup_list: typing.List[str] or typing.Tuple[str] or typing.Sequence[str]
+    :return: a generator containing all that are matched with ``x``
+    :rtype: typing.Generator or None
 
     **Examples**::
 
         >>> from pyhelpers.text import find_matched_str
 
-        >>> x = 'apple'
-
         >>> lookup_lst = ['abc', 'aapl', 'app', 'ap', 'ape', 'apex', 'apel']
-        >>> res = find_matched_str(x, lookup_lst)
-        >>> print(list(res))
+        >>> res = find_matched_str('apple', lookup_lst)
+        >>> list(res)
         []
 
-        >>> lookup_lst = ['abc', 'aapl', 'app', 'apple', 'ape', 'apex', 'apel']
-        >>> res = find_matched_str(x, lookup_lst)
-        >>> print(list(res))
+        >>> lookup_lst += ['apple']
+        >>> lookup_lst
+        ['abc', 'aapl', 'app', 'ap', 'ape', 'apex', 'apel', 'apple']
+
+        >>> res = find_matched_str('apple', lookup_lst)
+        >>> list(res)
         ['apple']
+
+        >>> res = find_matched_str(r'app(le)?', lookup_lst)
+        >>> list(res)
+        ['app', 'apple']
     """
 
-    assert isinstance(str_x, str), "`x` must be a string."
-    assert isinstance(lookup_list, collections.abc.Iterable), "`lookup_list` must be iterable."
+    if x is not None:
+        for y in lookup_list:
+            if re.match(x, y, re.IGNORECASE):
+                yield y
 
-    if str_x == '' or str_x is None:
-        return None
+
+def find_similar_str(x, lookup_list, n=1, ignore_punctuation=True, processor='difflib', **kwargs):
+    """
+    From among a sequence of strings, find ``n`` ones that are similar to ``x``.
+
+    :param x: a string-type variable
+    :type x: str
+    :param lookup_list: a sequence of strings for lookup
+    :type lookup_list: typing.List[str] or typing.Tuple[str] or typing.Sequence[str]
+    :param n: number of similar strings to return, defaults to ``1``;
+        if ``n=None``, the function returns a sorted ``lookup_list`` (in descending order of similarity)
+    :type n: int or None
+    :param processor: options include ``'difflib'`` (default) and ``'fuzzywuzzy'``
+
+        - if ``processor='difflib'``, the function relies on `difflib.get_close_matches`_
+        - if ``processor='fuzzywuzzy'``, the function relies on `fuzzywuzzy.fuzz.token_set_ratio`_
+
+    :type processor: str or None
+
+    :param ignore_punctuation: whether to ignore puctuations in the search for similar texts
+    :type ignore_punctuation: bool
+    :param kwargs: [optional] parameters of `difflib.get_close_matches`_ or
+        `fuzzywuzzy.fuzz.token_set_ratio`_, depending on ``processor``
+    :return: a string-type variable that should be similar to (or the same as) ``x``
+    :rtype: str or list or None
+
+    .. _`difflib.get_close_matches`:
+        https://docs.python.org/3/library/difflib.html#difflib.get_close_matches
+    .. _`fuzzywuzzy.fuzz.token_set_ratio`:
+        https://github.com/seatgeek/fuzzywuzzy
+
+    **Examples**::
+
+        >>> from pyhelpers.text import find_similar_str
+
+        >>> lookup_lst = ['Anglia',
+        ...               'East Coast',
+        ...               'East Midlands',
+        ...               'North and East',
+        ...               'London North Western',
+        ...               'Scotland',
+        ...               'South East',
+        ...               'Wales',
+        ...               'Wessex',
+        ...               'Western']
+
+        >>> str_similar = find_similar_str('angle', lookup_lst)
+        >>> str_similar
+        'Anglia'
+        >>> str_similar = find_similar_str('angle', lookup_lst, processor='fuzzywuzzy')
+        >>> str_similar
+        'Anglia'
+
+        >>> str_similar = find_similar_str('x', lookup_lst)
+        >>> str_similar  # None
+
+        >>> str_similar = find_similar_str('x', lookup_lst, processor='fuzzywuzzy')
+        >>> str_similar
+        'Wessex'
+    """
+
+    assert processor in ('difflib', 'fuzzywuzzy', None), \
+        "Options for `processor` include \"difflib\" and \"fuzzywuzzy\"."
+
+    m = len(lookup_list) if n is None else copy.copy(n)
+
+    if processor == 'difflib' or processor is None:
+        x_ = x.lower()
+        lookup_dict = {y.lower(): y for y in lookup_list}
+
+        if ignore_punctuation:
+            x_ = remove_punctuation(x_)
+            lookup_dict = {remove_punctuation(k): v for k, v in lookup_dict.items()}
+
+        sim_str_ = difflib.get_close_matches(word=x_, possibilities=lookup_dict.keys(), n=m, **kwargs)
+
+        if not sim_str_:
+            sim_str = None
+        else:
+            sim_str = lookup_dict[sim_str_[0]]
+
+    elif processor == 'fuzzywuzzy':
+        import fuzzywuzzy.fuzz
+
+        l_distances = [fuzzywuzzy.fuzz.token_set_ratio(s1=x, s2=a, **kwargs) for a in lookup_list]
+
+        if sum(l_distances) == 0:
+            sim_str = None
+        else:
+            if m == 1:
+                sim_str = lookup_list[l_distances.index(max(l_distances))]
+            else:
+                sim_str = [lookup_list[i] for i in np.argsort(l_distances)[::-1]][:m]
 
     else:
-        for y in lookup_list:
-            if re.match(str_x, y, re.IGNORECASE):
-                yield y
+        sim_str = None
+
+    return sim_str
 
 
 """ == Basic computation of textual data ===================================================== """
@@ -331,16 +341,15 @@ def calculate_idf(raw_documents, rm_punc=False):
 
     :param raw_documents: a series of documents
     :type raw_documents: pandas.Series
-    :param rm_punc: whether to remove punctuation from ``raw_documents``,
-        defaults to ``False``
+    :param rm_punc: whether to remove punctuation from ``raw_documents``, defaults to ``False``
     :type rm_punc: bool
     :return: term frequency (TF) of ``raw_documents``, and inverse document frequency
     :rtype: tuple
 
     **Examples**::
 
-        >>> import pandas
         >>> from pyhelpers.text import calculate_idf
+        >>> import pandas
 
         >>> raw_doc = pandas.Series(['This is an apple.',
         ...                          'That is a pear.',
@@ -348,14 +357,14 @@ def calculate_idf(raw_documents, rm_punc=False):
         ...                          'Hello world!'])
 
         >>> docs_tf_, corpus_idf_ = calculate_idf(raw_doc, rm_punc=False)
-        >>> print(docs_tf_)
+        >>> docs_tf_
         0    {'This': 1, 'is': 1, 'an': 1, 'apple': 1, '.': 1}
         1      {'That': 1, 'is': 1, 'a': 1, 'pear': 1, '.': 1}
         2    {'It': 1, 'is': 1, 'human': 1, 'being': 1, '.'...
         3                     {'Hello': 1, 'world': 1, '!': 1}
         dtype: object
 
-        >>> print(corpus_idf_)
+        >>> corpus_idf_
         {'This': 0.6931471805599453,
          'is': 0.0,
          'an': 0.6931471805599453,
@@ -372,14 +381,14 @@ def calculate_idf(raw_documents, rm_punc=False):
          '!': 0.6931471805599453}
 
         >>> docs_tf_, corpus_idf_ = calculate_idf(raw_doc, rm_punc=True)
-        >>> print(docs_tf_)
+        >>> docs_tf_
         0     {'This': 1, 'is': 1, 'an': 1, 'apple': 1}
         1       {'That': 1, 'is': 1, 'a': 1, 'pear': 1}
         2    {'It': 1, 'is': 1, 'human': 1, 'being': 1}
         3                      {'Hello': 1, 'world': 1}
         dtype: object
 
-        >>> print(corpus_idf_)
+        >>> corpus_idf_
         {'This': 0.6931471805599453,
          'is': 0.0,
          'an': 0.6931471805599453,
@@ -426,8 +435,8 @@ def calculate_tf_idf(raw_documents, rm_punc=False):
 
     **Examples**::
 
-        >>> import pandas
         >>> from pyhelpers.text import calculate_tf_idf
+        >>> import pandas
 
         >>> raw_doc = pandas.Series(['This is an apple.',
         ...                          'That is a pear.',
@@ -435,7 +444,7 @@ def calculate_tf_idf(raw_documents, rm_punc=False):
         ...                          'Hello world!'])
 
         >>> docs_tf_idf_ = calculate_tf_idf(raw_doc, rm_punc=False)
-        >>> print(docs_tf_idf_)
+        >>> docs_tf_idf_
         0    {'This': 0.6931471805599453, 'is': 0.0, 'an': ...
         1    {'That': 0.6931471805599453, 'is': 0.0, 'a': 0...
         2    {'It': 0.6931471805599453, 'is': 0.0, 'human':...
@@ -443,7 +452,7 @@ def calculate_tf_idf(raw_documents, rm_punc=False):
         dtype: object
 
         >>> docs_tf_idf_ = calculate_tf_idf(raw_doc, rm_punc=True)
-        >>> print(docs_tf_idf_)
+        >>> docs_tf_idf_
         0    {'This': 0.6931471805599453, 'is': 0.0, 'an': ...
         1    {'That': 0.6931471805599453, 'is': 0.0, 'a': 0...
         2    {'It': 0.6931471805599453, 'is': 0.0, 'human':...
@@ -477,7 +486,7 @@ def euclidean_distance_between_texts(txt1, txt2):
         >>> txt_1, txt_2 = 'This is an apple.', 'That is a pear.'
 
         >>> e_dist = euclidean_distance_between_texts(txt_1, txt_2)
-        >>> print(e_dist)
+        >>> e_dist
         2.6457513110645907
     """
 
@@ -520,11 +529,11 @@ def cosine_similarity_between_texts(txt1, txt2, cosine_distance=False):
         >>> txt_1, txt_2 = 'This is an apple.', 'That is a pear.'
 
         >>> cos_sim = cosine_similarity_between_texts(txt_1, txt_2)
-        >>> print(cos_sim)
+        >>> cos_sim
         0.6963106238227914
 
         >>> cos_dist = cosine_similarity_between_texts(txt_1, txt_2, cosine_distance=True)
-        >>> print(cos_dist)  # 1 - cos_sim
+        >>> cos_dist  # 1 - cos_sim
         0.3036893761772086
     """
 
@@ -573,12 +582,12 @@ def convert_md_to_rst(path_to_md, path_to_rst, verbose=False, pandoc_exe=None, *
     :param pandoc_exe: absolute path to 'pandoc.exe', defaults to ``None``
         (on Windows, use the default installation path - ``"C:\\Program Files\\Pandoc\\pandoc.exe"``)
     :type pandoc_exe: str or None
-    :param kwargs: optional parameters of `pypandoc.convert_file <https://github.com/bebraw/pypandoc>`_
+    :param kwargs: [optional] parameters of `pypandoc.convert_file <https://github.com/bebraw/pypandoc>`_
 
     **Example**::
 
-        >>> from pyhelpers.dir import cd
         >>> from pyhelpers.text import convert_md_to_rst
+        >>> from pyhelpers.dir import cd
 
         >>> dat_dir = cd("tests\\data")
 
